/*
 * RoombaBumpTurn 3.0
 * --------------
 * Implement the RoombaComm BumpTurn program in Arduino
 * A simple algorithm that allows the Roomba to drive around 
 * and avoid obstacles.
 * 
 * Arduino pin 4 (RX) is connected to Roomba TXD
 * Arduino pin 3 (TX) is connected to Roomba RXD
 * Arduino pin 2      is conencted to Roomba DD
 * 
 * Rewritten 28 January 2013 to use Arduino 1.0 SoftSerial,
 * act a little differently and use the hardware serial
 * port for debug transmission. (Dennis Clark)
 *
 * Updated 20 November 2006 with the idea of using two serial 
 * ports, one for debugging by Josh Heling, from his website
 * http://www.netfluvia.org/layer8
 *
 * Originally created 1 August 2006
 * copyleft 2006 Tod E. Kurt <tod@todbot.com>
 * http://hackingroomba.com/
 */

#include <SoftwareSerial.h>

int rxPin = 4;
int txPin = 3;
int ddPin = 2;
int ledPin = 13;

/*
*  Define number of sensor bytes and the 
*  byte array.
*/
#define BYTES 10
uint8_t sensorbytes[BYTES];

SoftwareSerial mySerial(rxPin,txPin);  // Roomba comm port

#define bumpright (sensorbytes[0] & 0x01)
#define bumpleft  (sensorbytes[0] & 0x02)

void setup() {
  uint8_t i;
  
  pinMode(ddPin,  OUTPUT);   // DD "wakeup" pin is an output
  pinMode(ledPin, OUTPUT);   // LED pin is an output
  
  /*
  *  Set up the software serial port to talk to the Roomba.
  */
  mySerial.begin(57600);

  /*
  *  Set up the hardware serial port for debugging.  Most of 
  *  the debug output is commented out so that it doesn't 
  *  slow down the response times reacting to bumper hits.
  *  To use them, uncomment the "Serial.print() methods.
  */
  Serial.begin(57600);
  
  Serial.println("Wake up Roomba\n");
  digitalWrite(ledPin, HIGH); // say we're alive
  digitalWrite(ddPin, HIGH);
  delay(100);
  digitalWrite(ddPin, LOW);
  delay(500);
  digitalWrite(ddPin, HIGH);
  
  /*
  * Grab whatever interesting stuff the Roomba transmits
  * and send it to debug port.
  */
  while(mySerial.available())
  {
    i = mySerial.read();
    Serial.write(i);
  }
  Serial.println();
  delay(2000);
  
  /*
  *  Set up Roomba to receive commands in 
  *  "Safe" mode.
  */
  Serial.println("set up ROI");
  mySerial.write((uint8_t)128);  // START
  delay(50);
  mySerial.write((uint8_t)130);  // CONTROL
  delay(50);
  digitalWrite(ledPin, LOW);  // say we've finished setup  
}

/*
*  This main loop is really simple.  At the top of each
*  pass through the loop ask for Sensor Packet 1 (10 bytes).
*  We are only looking at the first byte which gives the 
*  state of the bump sensors.
*  If we see a bumper response on the left or the right, 
*  act accordingly, otherwise just go forward.
*  Additionally, if any bumper is seen, we'll turn on the
*  corner brush motor so we can tell what is happening.
*/
void loop() {
  digitalWrite(ledPin, HIGH); // say we're starting loop
  UpdateSensors();
  //Serial.print("Sensor = ");
  //Serial.println(sensorbytes[0],HEX);
  //Serial.println("");
  //delay(1000);
  
  if(bumpleft) {
    DoBrush(1);
    GoBackward();
    delay(1000);
    SpinRight();
    delay(1000);
    DoBrush(0);
  }
  else if(bumpright) {
    DoBrush(1);
    GoBackward();
    delay(1000);
    SpinLeft();
    delay(1000);
    DoBrush(0);
  }
  GoForward();
  
  digitalWrite(ledPin, LOW);  // We've ended a loop
}

void DoBrush(uint8_t on) {
  // Turn side brush on or off
  if (on)
  {
    mySerial.write((uint8_t)138);
    mySerial.write((uint8_t)0x01);
  }
  else 
  {
    mySerial.write((uint8_t)138);
    mySerial.write((uint8_t)0x00);
  }
}

void GoForward() {
  mySerial.write((uint8_t)137);   // DRIVE
  mySerial.write((uint8_t)0x00);   // 0x00c8 == 200
  mySerial.write((uint8_t)0xc8);
  mySerial.write((uint8_t)0x80);
  mySerial.write((uint8_t)0x00);
}
void GoBackward() {
  mySerial.write((uint8_t)137);   // DRIVE
  mySerial.write((uint8_t)0xff);   // 0xff38 == -200
  mySerial.write((uint8_t)0x38);
  mySerial.write((uint8_t)0x80);
  mySerial.write((uint8_t)0x00);
}
void SpinLeft() {
  mySerial.write((uint8_t)137);   // DRIVE
  mySerial.write((uint8_t)0x00);   // 0x00c8 == 200
  mySerial.write((uint8_t)0xc8);
  mySerial.write((uint8_t)0x00);
  mySerial.write((uint8_t)0x01);   // 0x0001 == spin left
}
void SpinRight() {
  mySerial.write((uint8_t)137);   // DRIVE
  mySerial.write((uint8_t)0x00);   // 0x00c8 == 200
  mySerial.write((uint8_t)0xc8);
  mySerial.write((uint8_t)0xff);
  mySerial.write((uint8_t)0xff);   // 0xffff == -1 == spin right
}
void UpdateSensors() {
  int8_t  i = 0;
  mySerial.write((uint8_t)142);
  mySerial.write((uint8_t)1);  // sensor packet 1, 10 bytes

  while(!mySerial.available());  // wait for data to appear.
  
  while(mySerial.available()    // get 10 bytes of sensor data
  &&   (i < BYTES) ) 
  {
    int c = mySerial.read();
    if( c==-1 ) {
      for( int i=0; i<5; i ++ ) {   // say we had an error via the LED
        digitalWrite(ledPin, HIGH); 
        delay(50);
        digitalWrite(ledPin, LOW);  
        delay(50);
      }
    }
    sensorbytes[i++] = c;
  }    
  //Serial.print("Got ");Serial.print(i);Serial.println(" bytes");
  //i = 0;
  //while (i++ < BYTES)
  //{
  //  Serial.print(i,HEX); Serial.print(" ");
  //}
  //Serial.println();
}

